# PYTHON configuration file for class: CMSDAS11DijetAnalyzer.cc
# Description:  Example of simple EDAnalyzer for dijet mass & dijet spectrum ratio analysis
# Authors: J.P. Chou, Jason St. John
# Date:  01 - January - 2011
import FWCore.ParameterSet.Config as cms

process = cms.Process("Ana")
process.load("FWCore.MessageService.MessageLogger_cfi")

#############   Set the number of events #############
process.maxEvents = cms.untracked.PSet(
    input = cms.untracked.int32(1000)
)

############# Set the input for the analyzer #########
# Which input files to use
# 1: data, 10k events
# 2: MC resonance 1.2TeV
# 3: MC QCD only, no resonance
whichfiles = 1;

#############   Format MessageLogger #################
process.MessageLogger.cerr.FwkReport.reportEvery = 10

#############  This is how CMS handles output ROOT files #################
process.TFileService = cms.Service(
    "TFileService",
    fileName = cms.string("histos.root")
 )


if (whichfiles==1):
    thefileNames = cms.untracked.vstring('file:/uscms_data/d2/kalanand/dijet-Run2010A-JetMET-Nov4ReReco-9667events.root')
elif (whichfiles==2):
    thefileNames = cms.untracked.vstring('/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0026/6C0BC238-4247-DF11-81D7-E41F1318160C.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/FE3B276E-1C47-DF11-83B7-00215E22053A.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/FCD94068-2647-DF11-80FF-E41F131817F8.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/F843DD92-2647-DF11-9B8F-E41F13181668.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/F62E9060-1C47-DF11-B431-00215E221BC0.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/DA63FD75-1C47-DF11-A347-00215E221692.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/D802BE12-1D47-DF11-9FA1-00215E93D738.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/D65DA979-1C47-DF11-941C-00215E21DD56.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/D2647956-1C47-DF11-9C8E-00215E2216EC.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/D0A03150-1C47-DF11-ACE6-E41F1318099C.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/CC2D830B-1D47-DF11-966E-E41F13181498.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/CACB401B-1847-DF11-B9C5-E41F13181D00.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/CA85A868-2647-DF11-86BA-00215E221818.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/CA4A0A60-1C47-DF11-836E-00215E21D86A.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/CA070629-2747-DF11-819C-00215E222790.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/C0757277-1C47-DF11-90B7-00215E2212D2.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/BE19E200-1D47-DF11-A019-00215E22200A.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/AE023D13-1D47-DF11-892F-E41F13181688.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/AC98A079-1E47-DF11-8842-E41F13180A64.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/AA56B163-1C47-DF11-BF57-00215E21D570.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/9E440461-1C47-DF11-AC32-00215E21DBA0.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/9A160E70-1C47-DF11-AE68-00215E2208EE.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/96DF3203-1D47-DF11-849B-00215E21D948.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/9606A290-2647-DF11-A095-E41F13181AB4.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/9451B34F-1C47-DF11-B75C-E41F13181A5C.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/80CCEE69-1E47-DF11-9216-00215E2211F4.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/76B9DA6A-1C47-DF11-B13C-00215E222340.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/74489C67-2647-DF11-9B52-E41F13181588.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/6E8FD06A-1C47-DF11-A1DF-00215E21D9F6.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/6E872979-1C47-DF11-87DE-00215E21D540.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/646F4C0E-1D47-DF11-9C8E-00215E21D7C8.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/62D1695A-1C47-DF11-9C53-00215E21D786.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/5CAAC1A3-1B47-DF11-9B5B-00215E21DBBE.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/5A173FCA-2547-DF11-8F56-E41F131816A0.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/5670C812-1D47-DF11-8020-E41F13181890.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/52597D68-1E47-DF11-B7D1-E41F131816B4.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/4ECB6F7B-1E47-DF11-833D-E41F131816A0.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/4EB42E67-1C47-DF11-A1FA-00215E2205AC.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/4EB42E67-1C47-DF11-A1FA-00215E2205AC.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/4AEB1C64-1C47-DF11-BA6A-00215E221B48.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/48CCB914-1D47-DF11-B3D2-E41F13181044.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/3E489C57-1C47-DF11-A6EC-00215E93ED9C.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/3C04D1FE-1C47-DF11-8E76-E41F1318170C.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/3AE57490-2647-DF11-AA52-E41F13181CF8.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/38AC77A1-1B47-DF11-B015-E41F13181CA4.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/2AD2CC1E-1847-DF11-BCE3-00215E21D57C.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/2A4DFD09-1D47-DF11-9D16-00215E2223D6.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/2A407379-1C47-DF11-A7F2-00215E21DAF2.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/289FB808-1D47-DF11-A091-00215E222808.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/28160333-1847-DF11-BAD5-00215E21D702.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/220A4374-1C47-DF11-9FA1-00215E21DF18.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/20D4E62F-1847-DF11-8D41-00215E2222A4.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/1C8DD3A3-1B47-DF11-BAF8-E41F1318168C.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/169CCA06-1D47-DF11-B2C0-00215E221EEA.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/102D9F52-1C47-DF11-B437-00215E22181E.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/0EB6DB6E-1E47-DF11-8D0B-00215E93DCFC.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/08056A58-1C47-DF11-A42D-00215E21DAAA.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0025/06E9D555-1C47-DF11-9E40-00215E2219E6.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0024/E2CAFC76-0C47-DF11-9784-00215E93E7DC.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0024/94318864-1147-DF11-B2FD-E41F131815B8.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0024/66451B78-0C47-DF11-8CB0-00215E93EE44.root',
                                      '/store/mc/Spring10/Qstar_DiJet1200/GEN-SIM-RECO/START3X_V26_S09-v1/0024/5A3665E0-1047-DF11-9537-00215E22175E.root'
                                      )
elif (whichfiles==3):
    thefileNames = cms.untracked.vstring()    

#############   Define the source file ###############
process.source = cms.Source("PoolSource",
                            fileNames = thefileNames
                            )          

if (whichfiles ==3):
        thefileNames.extend( [
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0024/F4FDEF3F-9CF1-DF11-9AC7-00304867FD2B.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0024/DE3FF3C8-A2F1-DF11-AF5A-0030483355A8.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0024/C84D7DC5-CEF1-DF11-BF16-0002C90A3696.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0024/C0E8D0BD-A4F1-DF11-805E-00304867FEBB.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0024/BE7BD0DF-D8F1-DF11-B336-00304867FD4F.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0024/9E71C0AA-A2F1-DF11-976B-00304867FF17.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0024/861C09C8-A4F1-DF11-A452-00304867FD67.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0024/6A6F80C3-9BF1-DF11-A1D2-0023546BA20F.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0024/626D08D6-CEF1-DF11-A5DB-00E081237A11.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0024/48ED637F-D2F1-DF11-88AB-00238BCE45B0.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0024/2E8F9757-EEF1-DF11-A6AB-001A9243D5C7.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0024/1241B8C7-C2F1-DF11-BF17-0002C90A344C.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/FEEB3B9A-7BF1-DF11-858B-001A9243D5E7.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/FEAA5C75-B1F1-DF11-92F7-0002C90A370A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/FC7E715F-9FF1-DF11-A5F9-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/FAFB83A2-56F1-DF11-A05B-0015174F088C.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/FAF0F14D-75F1-DF11-A206-001A9243D537.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/FAC426AD-80F1-DF11-A913-003048322B3E.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/FAAB6CBC-49F1-DF11-AEDA-0019B9CADC3D.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/FA566CF9-B6F1-DF11-AE12-0002C90A34A8.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/FA0D26DA-88F1-DF11-8FA8-00238BCE45EC.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/F8AB2811-5EF1-DF11-A71B-A4BADB1E65B1.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/F62D5C87-B1F1-DF11-B1D6-0002C90B742A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/F49A8284-5FF1-DF11-B548-0025901D0C54.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/F418C6FF-81F1-DF11-85BB-00238BBD7674.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/F2AF5FA2-56F1-DF11-BA54-0015174ED2F2.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/F2A07C3E-57F1-DF11-9F64-A4BADB1C5D42.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/F26EB9A8-9BF1-DF11-ADE3-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/F248E247-9AF1-DF11-8FAC-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/F0EB31E7-5FF1-DF11-8208-A4BADB1E6796.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/EEAD2A29-A6F1-DF11-8D47-0002C90A36AA.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/EC8E1B98-A8F1-DF11-BEC7-0002C90A34A2.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/E820F4EF-4EF1-DF11-BC2F-00E08120B64F.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/E65B3AD3-9BF1-DF11-9A32-0002C90A3414.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/E6206705-82F1-DF11-A572-00238BCE463C.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/E4CD2AB7-61F1-DF11-804E-842B2B019EA1.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/E4A87E79-57F1-DF11-9BFA-842B2B1807B2.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/E27C3FD4-9BF1-DF11-BD20-0002C90A3414.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/E2772FF1-B6F1-DF11-B762-0002C90A3642.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/E24E4111-5EF1-DF11-97EA-842B2B42B536.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/E0643905-9FF1-DF11-9EEE-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/E0434024-59F1-DF11-899D-842B2B17E35B.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/DE61E846-A4F1-DF11-9490-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/DC5F280C-BCF1-DF11-B624-0002C90A34A2.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/DA9677AE-77F1-DF11-AC2C-0025901D08B8.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/D8203073-B3F1-DF11-AC88-0002C90A34A2.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/D6547615-9EF1-DF11-8B7A-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/D04A2B71-B1F1-DF11-ADCC-0002C90A370A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/D02E718B-68F1-DF11-BCA3-842B2B1810D5.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/CE85F8CC-B8F1-DF11-86B0-0002C90A3526.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/C89D3872-7BF1-DF11-8556-00304834BB58.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/C841C4C9-53F1-DF11-8765-001731EB1E48.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/C63AAE0D-89F1-DF11-B8AA-0023546BA347.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/C4CC68C9-A0F1-DF11-9FB2-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/C2F099F6-A7F1-DF11-95AB-0002C90A3562.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/C2B5986C-AAF1-DF11-9839-0002C90B7FCA.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/C28CA099-49F1-DF11-8F12-001EC94B4F72.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/C0C301F9-99F1-DF11-90FD-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/C0609469-9FF1-DF11-BC6E-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/C00F51B8-85F1-DF11-AE3A-001A92971CDD.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/BEFA2F14-A8F1-DF11-90FC-0002C90A36FC.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/BE804392-B3F1-DF11-973F-0002C90A3642.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/BE4F996C-AAF1-DF11-9044-0002C90B7FCA.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/BADD38C8-98F1-DF11-BF02-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/BADB5A25-C1F1-DF11-9759-0002C90B3976.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/BA773002-7DF1-DF11-A2C8-E0CB4E2E4E6A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/BA762614-80F1-DF11-B41A-001A92971C6D.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/BA0EB2F0-5CF1-DF11-8E11-0025901D08EC.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/B8E7BFD5-7EF1-DF11-9696-001CC05CC464.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/B8C3A2DE-A2F1-DF11-AF2C-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/B8631A72-A2F1-DF11-8D14-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/B4B3B985-62F1-DF11-9C98-001731EB1F36.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/B0F888A0-55F1-DF11-A646-0026B93B21AE.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/B0DFB321-8BF1-DF11-A5A0-00304865C331.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/B0622FD6-96F1-DF11-819A-0002C90A3698.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/AEB0F0D7-A0F1-DF11-A714-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/ACA8612C-A4F1-DF11-AC11-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/AC532319-46F1-DF11-9905-00E081416320.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/AAC817A8-B8F1-DF11-A9DC-0002C90A3526.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/AABE3FF2-5FF1-DF11-9AC7-A4BADB1C5D42.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/A8FD6976-6AF1-DF11-B218-A4BADB22B643.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/A8556918-9EF1-DF11-BD12-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/A8033BD7-9BF1-DF11-B629-0002C90A3414.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/A67557E7-A9F1-DF11-8619-0002C90B7412.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/A63C3F9F-9DF1-DF11-845C-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/A433FC40-9AF1-DF11-B99A-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/A42F1DC0-53F1-DF11-9C58-842B2B42BC3A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/9C70BD05-A8F1-DF11-9766-0002C90A36FC.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/9AA1694C-81F1-DF11-A62A-00238BCE4616.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/9A5051AC-44F1-DF11-ACFB-842B2B0A39C8.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/98E74C3E-77F1-DF11-80B2-003048F59728.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/98D16CAB-50F1-DF11-B662-0019B9CAFE71.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/98B0F425-A6F1-DF11-940D-0002C90A36B8.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/96C25F29-82F1-DF11-A832-003048F5970A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/96BE27D3-5FF1-DF11-816D-00E081551CFC.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/96B30CFB-99F1-DF11-96B8-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/961DDD24-A4F1-DF11-B169-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/9601D204-82F1-DF11-844D-003048344BF8.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/94761063-9FF1-DF11-9887-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/9452E9F2-5FF1-DF11-B5E9-A4BADB22A4AE.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/94399AB8-98F1-DF11-8D13-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/8EF10C46-B9F1-DF11-9101-0002C90A3642.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/8EEFB29F-9DF1-DF11-AC11-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/8C34DCFE-81F1-DF11-A836-003048F5B69A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/8AFC6334-A4F1-DF11-8231-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/8ADCE3B2-89F1-DF11-8C34-00304832293E.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/8A5A3212-5EF1-DF11-96FC-0026B93A0356.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/86F07028-A4F1-DF11-B25F-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/849007DE-A5F1-DF11-AB39-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/842A18A9-B7F1-DF11-BFC0-0002C90A3428.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/82C0303A-57F1-DF11-80D2-A4BADB1E6F7A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/820CCAC9-91F1-DF11-B8D1-00304867FEAF.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/80914757-5EF1-DF11-82A5-0026B937D207.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/7CE18D67-98F1-DF11-9BAF-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/7C569530-B7F1-DF11-A763-0002C90A3642.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/7A9F190E-5FF1-DF11-BA2E-0025901D08E8.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/7A34CE14-82F1-DF11-87BB-001A9243D62B.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/78E55DB3-B8F1-DF11-A028-0002C90A344C.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/7889EEE3-4EF1-DF11-8E78-842B2B17F73D.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/7835D19C-9DF1-DF11-A533-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/76BD67F8-7FF1-DF11-B1DD-001A9243D57D.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/74CE1485-8CF1-DF11-A5B8-00304867FE1B.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/74CCF51B-9EF1-DF11-A595-0002C90A36AA.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/745D968E-68F1-DF11-8D25-0013D4892034.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/721DA1F6-5FF1-DF11-B379-842B2B1807B2.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/70DC7FA9-61F1-DF11-B4BE-842B2B42D35D.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/70CB626E-B1F1-DF11-9DA7-0002C90A370A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/6EB0FF1C-A6F1-DF11-926A-0002C90A36AA.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/6E0F34FD-55F1-DF11-ADC2-003048F59728.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/6AA654D1-A0F1-DF11-B16C-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/688F037F-62F1-DF11-B225-00093D122E06.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/643C5C86-B3F1-DF11-93E2-0002C90A34A8.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/62157FEC-99F1-DF11-A13C-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/606196D5-BDF1-DF11-AD1B-0002C90A3696.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/5E894CD5-96F1-DF11-B89A-0002C90A3698.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/5C724B61-52F1-DF11-AF07-003048339B04.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/5C1AFC79-B3F1-DF11-A8C5-0002C90B7F2A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/5A2A34B8-57F1-DF11-B579-0025901D08D6.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/5A0FDE19-C1F1-DF11-BA5B-0002C90A36AA.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/56A945CB-8AF1-DF11-AE2C-00304867FF23.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/560FFA77-50F1-DF11-8F59-00304865C338.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/54D5A978-B3F1-DF11-9D42-0002C90B3974.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/52F2F50D-A8F1-DF11-BFAE-0002C90A36FC.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/52DEED70-52F1-DF11-802B-0015174ED5B2.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/5250A0AD-9BF1-DF11-87CE-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/523D7BA9-55F1-DF11-BAFA-842B2B17F557.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/521868E8-4EF1-DF11-85A9-842B2B17E8C6.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/5062DB3D-52F1-DF11-A1AF-0019B9CB020F.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/504A3591-49F1-DF11-B7A2-0026B937D37D.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/501E0B9C-76F1-DF11-928C-00304867FDAF.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/4EDF877A-A2F1-DF11-BDBC-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/4E35E675-57F1-DF11-8497-842B2B1815B3.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/4CCB4244-61F1-DF11-987B-00238B8A3CEE.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/4CB0604A-71F1-DF11-8B09-A4BADB1E6055.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/4C33CF4B-6AF1-DF11-A011-001731EF5DA9.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/40CA0154-9AF1-DF11-8C15-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/40A3382B-A4F1-DF11-8E31-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/409FC06F-7CF1-DF11-858E-001A92971C6D.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/401E8B7D-57F1-DF11-A57F-0026B93AA8FF.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/3E90D640-A1F1-DF11-8520-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/3E6B99C6-A0F1-DF11-9F10-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/3E279653-BAF1-DF11-B92E-0002C90A36B8.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/3C65E34B-98F1-DF11-A784-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/3A7BA10B-84F1-DF11-B4A4-0026180A8746.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/38B9C6B0-55F1-DF11-80AB-0026B939DCF3.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/38455B25-61F1-DF11-9970-003048770C64.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/365B8EE2-A2F1-DF11-82DF-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/36437AA5-61F1-DF11-B9F8-0026B93785F5.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/360DD1F4-5FF1-DF11-ADDE-842B2B18178A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/32685A03-6CF1-DF11-AE08-001731EB1E10.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/2CE14BAD-49F1-DF11-800B-0019B9CB01E8.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/2C482D7A-B1F1-DF11-83D7-0002C90B742A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/2C274A41-8FF1-DF11-B84A-485B3919F14E.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/2A9DF6AE-56F1-DF11-8F19-0013D4892044.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/2A692A5C-9FF1-DF11-B03D-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/2A65F390-5FF1-DF11-8A05-00238BCE4618.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/283BD49F-55F1-DF11-AF2C-842B2B17E35B.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/2834A370-44F1-DF11-80AC-A4BADB1C4493.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/24D6F864-57F1-DF11-A3D1-0015174ED4F6.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/2044A994-B7F1-DF11-9DFA-0002C90A3428.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/1EAFE1DF-A2F1-DF11-93E6-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/1AF8996C-AAF1-DF11-A1FC-0002C90B7FCA.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/1A9D4567-AAF1-DF11-B8F5-0002C90B7FCA.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/1A292E53-5EF1-DF11-AFB6-842B2B17E35B.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/1888E33C-80F1-DF11-B6DA-0025901D08B8.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/186738F0-A7F1-DF11-823E-0002C90A355A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/16C28946-98F1-DF11-A8A6-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/140D23D6-4EF1-DF11-A384-0026B937D37D.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/12B87E35-9AF1-DF11-8772-00304867FEBB.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/127B5B49-82F1-DF11-B5B6-003048F5B2AC.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/1075A5A9-44F1-DF11-9C2F-0026B939EA8A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/105EB874-82F1-DF11-8443-00304865C465.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/0E9B5859-5EF1-DF11-8C50-842B2B17E3BA.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/0E572BC2-B1F1-DF11-8D67-0002C90B742A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/0CD7CB4A-6AF1-DF11-8507-001731EF5E73.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/0C91FEF6-99F1-DF11-9A15-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/0C1C347A-92F1-DF11-9476-003048322A48.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/0A1092A2-56F1-DF11-A915-0015174F0A98.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/0663D14B-6AF1-DF11-88E8-001731EB1DE6.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/0409D2A4-85F1-DF11-813E-00304867FD77.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/024780B8-9BF1-DF11-BC92-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/0222799A-68F1-DF11-8F9B-0015F2399187.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/00E2E93E-A4F1-DF11-9D71-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/00C17C88-57F1-DF11-BD7D-003048344C1A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0023/00984235-A4F1-DF11-8456-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/FCCA7CB6-91F1-DF11-B5F4-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/FC77E073-87F1-DF11-903D-0002C90A3698.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/FC31FB20-29F1-DF11-8554-0026B93785EC.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/FAB223CA-96F1-DF11-A262-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/F6DA8E64-8BF1-DF11-8C9C-0002C90B7426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/F293382A-76F1-DF11-B126-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/F0F7A7C7-2AF1-DF11-A7BE-0026B939DCF3.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/F01B5EB8-91F1-DF11-89C0-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/EC589203-89F1-DF11-92E1-0002C90A36E2.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/E8866428-86F1-DF11-8E96-1CC1DEEB8798.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/E6D74457-73F1-DF11-93BC-842B2B17EA37.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/E4C0EC7B-1FF1-DF11-A409-001EC94BA169.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/E45934C1-8FF1-DF11-9AC3-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/E26C1402-8BF1-DF11-BB1A-0002C90A3492.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/DECD28ED-88F1-DF11-98F6-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/DCA47526-29F1-DF11-BC8E-0019B9CABE2A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/DC6FE33B-76F1-DF11-85A9-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/DC1F5FC7-91F1-DF11-8F73-0002C90A3698.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/DA730F72-44F1-DF11-BE2E-0026B937D38F.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/DA314767-8BF1-DF11-BF3B-0002C90B7426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/D8E03257-3FF1-DF11-978F-0026B93A0356.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/D6D2C3DE-36F1-DF11-A383-A4BADB1C5D42.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/D63E2958-41F1-DF11-B663-842B2B17E35B.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/D2F467AF-8FF1-DF11-8115-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/D2DF5DDC-36F1-DF11-85FB-0026B9392629.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/D29280BF-3CF1-DF11-B070-842B2B17E8C6.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/D04C9821-29F1-DF11-AE9B-842B2B019EE5.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/D010CFD3-8FF1-DF11-88F2-0002C90B39A0.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/CE21F080-71F1-DF11-8840-0002C90A3408.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/CA8FD072-7BF1-DF11-B759-0002C90A3402.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/C847FE89-3FF1-DF11-881D-842B2B42B584.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/C6D3B9C9-96F1-DF11-AFB1-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/C4C9380A-89F1-DF11-9D69-0002C90B39A0.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/C2B0006A-95F1-DF11-B151-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/C243A1D0-23F1-DF11-9CD4-0026B93AA8FF.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/C2305DF1-7CF1-DF11-8B0F-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/BAE1D8FC-88F1-DF11-B896-0002C90B7F2E.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/BA032B27-86F1-DF11-9445-1CC1DEEB8798.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/B8AFB458-30F1-DF11-87AF-00E081237305.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/B48031B8-22F1-DF11-A8A5-842B2B42B536.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/AE5B4629-86F1-DF11-8E28-0002C90B7F5E.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/AC9CE5F8-3CF1-DF11-A9F3-842B2B42B2B1.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/A8E34E6B-87F1-DF11-A032-0002C90A3698.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/A00F3767-75F1-DF11-A5EE-0002C90A3562.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/98F54F69-95F1-DF11-9C75-0002C90B7488.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/9618919C-91F1-DF11-8D5D-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/944A5A3D-93F1-DF11-AC75-0002C90A36AA.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/924B40CC-83F1-DF11-837B-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/8E54CFF3-36F1-DF11-85A7-842B2B1812E7.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/8C1C52DD-36F1-DF11-B96A-842B2B180CC3.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/8845A7CD-94F1-DF11-9669-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/825261BA-32F1-DF11-A5B3-0026B93A0356.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/824328D2-94F1-DF11-B96C-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/7E209BCF-96F1-DF11-A95A-0002C90A3698.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/7C3E2B52-22F1-DF11-8E47-A4BADB1E763D.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/7C2624D6-8FF1-DF11-8EFE-0002C90A3462.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/78DD9071-75F1-DF11-BBA0-0002C90A36FC.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/7252AD61-95F1-DF11-A467-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/720C4FD1-83F1-DF11-82F4-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/6EC0F5C1-3CF1-DF11-B169-A4BADB1E67BA.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/6CF8B100-89F1-DF11-95E3-0002C90A36B8.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/60E48BBD-85F1-DF11-80C0-0002C90B741E.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/5E941A31-76F1-DF11-AF15-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/5CC1127E-1FF1-DF11-BAA4-842B2B17EE7F.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/56D61A14-86F1-DF11-A71B-0002C90B7F5E.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/5248729D-96F1-DF11-8054-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/50E8FFE3-83F1-DF11-B2AF-0002C90A3698.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/4C56F8F7-3AF1-DF11-AC2A-842B2B42BCF8.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/4A8C7C7C-71F1-DF11-91CD-0002C90B741E.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/46193CFC-8AF1-DF11-86EE-0002C90A3408.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/441DA07F-33F1-DF11-86F3-842B2B1811D7.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/44050E3E-29F1-DF11-B8DD-842B2B17F73D.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/3CC70D1F-93F1-DF11-864B-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/3AD4AFAC-8FF1-DF11-A2DD-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/3A2FA952-22F1-DF11-A5A7-0026B937D268.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/38654AB8-32F1-DF11-B996-842B2B17367A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/32F1ED0D-8BF1-DF11-897E-0002C90A3492.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/32C2EF83-1FF1-DF11-84BA-0019B9CABE34.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/32AEF16E-87F1-DF11-95CD-0002C90A3698.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/2ECBDB8B-3FF1-DF11-AF7A-A4BADB1E6796.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/2E4D12C2-3CF1-DF11-A8C9-0026B937C6C9.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/2E3F0FFF-88F1-DF11-99A9-0002C90A3414.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/2872C171-87F1-DF11-8462-0002C90A36AA.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/26F0CDB3-8FF1-DF11-A628-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/223727C7-85F1-DF11-BE56-0002C90A3408.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/1E4470C6-2AF1-DF11-B08F-A4BADB1E6602.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/1C068132-93F1-DF11-8D84-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/182317A3-82F1-DF11-8476-0002C90B3974.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/1657743F-93F1-DF11-AFAC-0002C90A36AA.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/161AFE9B-7BF1-DF11-85DA-0002C90A3402.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/14AC342C-93F1-DF11-971E-0002C90A3426.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/1432AAB5-22F1-DF11-9360-0026B937D3B3.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/126F6CB9-3AF1-DF11-9C0B-842B2B42B2B1.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/10427462-95F1-DF11-AAB2-0002C90A3690.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/0C8A206C-91F1-DF11-AAFD-0002C90B743A.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/0AEF6E06-41F1-DF11-87E8-A4BADB1E763D.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/063B0197-3AF1-DF11-9FC8-842B2B17F557.root',
        '/store/mc/Fall10/QCD_Pt-15to3000_TuneD6T_Flat_7TeV-pythia6/GEN-SIM-RECO/START38_V12-v1/0022/02E62CA9-91F1-DF11-812A-0002C90B743A.root'
        ] )


##-------------------- Communicate with the DB -----------------------
process.load('Configuration.StandardSequences.Services_cff')
process.load('Configuration.StandardSequences.FrontierConditions_GlobalTag_cff')
process.GlobalTag.globaltag = 'START38_V13::All'

#############   Include the jet corrections ##########
process.load("JetMETCorrections.Configuration.DefaultJEC_cff")
# set the record's IOV. Must be defined once. Choose ANY correction service. #

if whichfiles == 1:
    correctionsToGet = "ak7CaloL2L3Residual"
else:
    correctionsToGet = "ak7CaloL2L3"

#############   Correct Calo Jets on the fly #########
process.dijetAna = cms.EDAnalyzer("CMSDAS11DijetTestAnalyzer",
                                  jetSrc = cms.InputTag("ak7CaloJets"),
                                  vertexSrc = cms.InputTag("offlinePrimaryVertices"),
                                  jetCorrections = cms.string(correctionsToGet),
                                  innerDeltaEta = cms.double(0.7),
                                  outerDeltaEta = cms.double(1.3),
                                  JESbias = cms.double(1.0)
)

#############   Path       ###########################
process.p = cms.Path(process.dijetAna)



